/*
  file:         Alg_elements.hh
  created:      Sekhar Muddana 1990
  modified:     Pasha Zusmanovich feb 18,20-21,24,mar 1-2,8-10,13 2016
  modification: - C++ file extensions
				- templated Scalar
				- Alg_element turned to c++ map
*/

#ifndef __ALG_ELEMENTS_H
#define __ALG_ELEMENTS_H

#include <map>
#include "Build_defs.hh"

template<typename T> struct Alg_element
{
  typedef std::map<Basis, T> type;  // basis -> coef
};



// RETURNS: 1 if all basis coef of Alg_element *p are zeroes, 0 otherwise
template<typename T> bool IsZeroAE (const typename Alg_element<T>::type& p)
{
  typename std::map<Basis, T>::const_iterator i;
  for (i = p.begin(); i != p.end(); i++) 
	{
      if ((i->first != 0) && (i->second != (T)0)) 
		{
		  return (0);
		}
    }
  return (1);
}




template<typename T> inline void clear_zeros (typename Alg_element<T>::type& ae)
{
  typename std::map<Basis, T>::iterator i;
  for (i = ae.begin(); i != ae.end(); ) 
	{
      if ((i->first == 0) || (i->second == (T)0))
		{
		  ae.erase (i++);
		} 
	  else 
		{
		  i++;
		}
    }
}




// Multiply Alg_element *p with scalar x.
//     *p = *p * x.                      
// RETURNS: 1 if successfull, 0 otherwise.
// called once from IsIdentity() ?
template<typename T> void ScalarMultAE (const T& x, 
										typename Alg_element<T>::type& p)
{
  if (x == (T)1)
	{
	  return;
	}
  if (x == (T)0) 
	{
	  p.clear();
    }
  else 
	{
	  typename std::map<Basis, T>::iterator i;
      for (i = p.begin(); i != p.end(); i++) 
		{
		  i->second = x * (i->second);
		}
	  clear_zeros<T> (p);
    }
} // end of ScalarMultAE()





template<typename T> inline void SetAE (typename Alg_element<T>::type& p, 
										Basis b, T x) 
{
  if (x != (T)0) 
	{
	  p[b] = x;
	} 
  else 
	{
	  p.erase (b);
	}
}



template<typename T> inline T GetAE (const typename Alg_element<T>::type& p, 
									 const Basis b)
{
  typename std::map<Basis, T>::const_iterator i = p.find (b);
  return ((i != p.end()) ? (i->second) : (T)0);
}



#endif // __ALG_ELEMENTS_H
// end of Alg_elements.hh
