/*
  file:         Basis_table.cc
  purpose:      routines dealing with Basis Table
  created:      David P Jacobs May 1990
  modified:     pasha jan 4 2019
  modification: removed unnecessary header; cleanup
*/

//#include <setjmp.h>
#include <stdlib.h>
#include <iostream>
#include "albert.hh"

#include "Basis_table.hh"

#ifdef DMALLOC
#include <dmalloc.h>
#endif

bool InitializeDegtoBasisTable (int);
void UpdateDegToBasisTable (int, Basis);

static int Deg_of_last_basis;
Deg_to_basis_rec* Deg_to_basis_table = NULL;



// Initialize the Basis Table (to zeroes) and Nextbasistobefilled (to 1)
// RETURNS: 1 if successfull, 0 otherwise
bool Albert::CreateBasisTable (const int Target_degree)
{
  this->Basis_table.clear();

  // first entry is expected to be zero
  BT_rec br;
  br.left_factor  = 0;
  br.right_factor = 0;
  br.type = 0;
  Basis_table.push_back (br);

  if (Deg_to_basis_table != NULL)
	{
	  free (Deg_to_basis_table);
	}
  if (InitializeDegtoBasisTable (Target_degree) != 1)
	{
	  return (0);
	}
  Deg_of_last_basis = 0;
  return (1);
}

    

bool InitializeDegtoBasisTable (const int Target_degree)
{
  Deg_to_basis_table = 
	(Deg_to_basis_rec*)(Mymalloc (Target_degree * sizeof(Deg_to_basis_rec)));
  assert_not_null(Deg_to_basis_table);
  for (int i=0; i < Target_degree; i++) 
	{			
	  Deg_to_basis_table[i].first_basis = 0;
	  Deg_to_basis_table[i].last_basis  = 0;
    }
  return (1);
}



// Enter a New Basis into Basis Table.
// GLOBALS MODIFIED:                                 
//     Basis_table -- New Basis is entered.          
//     Nextbasistobefilled -- Incremented.           
// args: Left_factor -- of the new Basis.                    
//       Right_factor -- of the new Basis.                     
//       Type -- of the new Basis.                               
// RETURNS: Basis Number of the new Basis entered.                  
Basis Albert::EnterBasis 
(const Basis Left_factor, const Basis Right_factor, const Name cur_type)
{
  Basis bn = Basis_table.size();
  BT_rec br;
  br.left_factor  = Left_factor;
  br.right_factor = Right_factor;
  br.type = cur_type;
  Basis_table.push_back (br);
  UpdateDegToBasisTable (GetDegreeName (cur_type), bn);
  return (bn);
}



void UpdateDegToBasisTable (const int Deg, const Basis Cur_basis)
{
  if (Deg > Deg_of_last_basis) 
	{
	  Deg_to_basis_table[Deg-1].first_basis = Cur_basis;
	  Deg_to_basis_table[Deg-1].last_basis  = Cur_basis;
	  Deg_of_last_basis = Deg;
    }
  else
	{
	  (Deg_to_basis_table[Deg-1].last_basis)++;
	}
}

Basis BasisStart (Degree Deg)
{
  return (Deg_to_basis_table[Deg-1].first_basis);
}


Basis BasisEnd (const Degree Deg)
{
  return (Deg_to_basis_table[Deg-1].last_basis);
}


// RETURNS: Type of the Basis element B
Name Albert::GetType (const Basis B) const
{
  return (this->Basis_table[B].type);
}

// end of Basis_table.cc
