/*
  file:         Build.cc
  purpose:      Implement the Build Command.
                Reads the sparse global variable to determine
      	        whether the traditional or sparse code should be used
                in the SolveEquations routine.
  created:      David P. Jacobs May 1990
  modified:     pasha sep 8-9 2015,feb 9,11-13,18-19,21-22,27,mar 10,13-14 2016
  modification: - corrected formatting in printing
                - cleanup
				- support for mingw and dmalloc
				- Type converted to vector
				- attempt to template Scalar
				- c++ file ext.
*/

#include "albert.hh"
#include "Build.hh"

#ifdef DMALLOC
#include <dmalloc.h>
#endif


// All Degree 1 Basis i.e Generators are entered into Basis Table
void Albert::InstallDegree1()
{
  Name n;
  Basis begin_basis;
  Basis end_basis = 0;
  Type temp_type;
  GetNewType (temp_type);
  const unsigned int len = GetTargetLen();
  unsigned int j;
  for (unsigned int i=0; i<len; i++) 
	{
	  for (j=0; j<len; j++)
		{
		  temp_type[j] = 0;
		}
	  temp_type[i] = 1;
	  n = TypeToName (temp_type);
	  begin_basis = this->Basis_table.size();
	  EnterBasis (0, 0, n);
	  end_basis = this->Basis_table.size() - 1;
	  UpdateTypeTable (n, begin_basis, end_basis);
    }
  this->Current_dimension = end_basis;
} // end of Albert::InstallDegree1()




void _print_density (const double density)
{
  std::cout << "Last Matrix " << std::fixed << std::setprecision(1) << density
			<< "% dense." << std::endl;
}


// end of Build.cc
