/*
  file:         Mult_table.hh
  created:      Sekhar Muddana May 1990
  modified:     Pasha Zusmanovich feb 18,20-21,24,27,mar 2,7,9-10,12 2016
  modification: - C++ file extensions
                - templated Scalar
				- include once
				- removed def of MTB_SIZE
				- more c++ structures
*/

#ifndef __MULT_TABLE_H
#define __MULT_TABLE_H

#include "assert_not_null.hh"
#include "Memory_routines.hh"


template<typename T> struct Term
{
  T coef;
  Basis word;
};

// A block of Terms.
// Each of these point to next block of terms.
// Linear list of terms in abstract sense.
// Storage unit for building Mult.Table.
template<typename T> struct Terms_block 
{
  Term<T>* terms;
  struct Terms_block<T>* next;
};



// Recursively Free the memory occupaid by all the Terms_blocks
// args: P -- Pointer to First_terms_block
//       X -- For Stats
// RETURNS: 1 if successfull, 0 otherwise
template<typename T> bool FreeTermsBlocks (Terms_block<T>* P, unsigned int* X)
{
  assert_not_null(P);
  *X = *X + 1;
  if (P->next == NULL)
	{
      if (P->terms != NULL)
		{
		  free (P->terms);
		}
      free (P);
    }
  else
	{
      FreeTermsBlocks<T> (P->next, X);
      if (P->terms != NULL)
		{
		  free(P->terms);
		}
      free (P);
    }
  return (1);
} // end of FreeTermsBlocks()




template<typename T> unsigned int TermsListLength 
(std::vector<std::pair<T, Basis> >& Tl)
{
  unsigned int i = 0;
  while (Tl[i].first != (T)0) 
	{
	  i++;
    }
  return (i+1);
}


#endif // __MULT_TABLE_H
// end of Mult_table.hh
