/*
  file:         Po_prod_bst.cc
  purpose:      these routines used by Po_parse_poly.cc
                to set up search tree of productions
  created:      Jagdish Thurimella and Sekhar Muddana circa 1990
  modified:     Pasha Zusmanovich feb 18,mar 1 2016
  modification: cleanup; c++ file ext.; commented out debug code
*/

#include <ctype.h>
#include <stdio.h>
#include <string.h>
#include "Memory_routines.hh"

#include "Po_prod_bst.hh"

int EQUAL (char[], char[]);
int GT    (char[], char[]);
int LT    (char[], char[]);



// Insert the production at the right place in the binary tree, after allocating
// space. Key is the string.
// This routine is called to store all the production as a part of 
// initialization. Called from Init_prod_tree() in the module Po_parse_poly.c
// MODIFIES: *Node_ptr -- insert a new node into the tree.
// args:
//     X -- string, is copied into a field in the new node.       
//          The string equivalent of Handle of the production,    
//          obtained by gluing integers together as a string.     
//     Left -- Non terminal of the production                     
//     Pnum -- Production number.                                 
// RETURNS: *Node_ptr -- Pointer to the tree of productions.
PROD_TREEPTR Prod_insert (char X[], int Left, int Pnum, PROD_TREEPTR* Node_ptr)
{
  if (*Node_ptr == NULL) 
	{ 
	  (*Node_ptr) = prod_talloc();
	  strcpy((*Node_ptr)->rhs_str, X);
	  (*Node_ptr)->lhs     = Left;
	  (*Node_ptr)->prodnum = Pnum;
	  (*Node_ptr)->left	   = NULL;
	  (*Node_ptr)->right   = NULL;
	  return (*Node_ptr);
    }
  else if (LT(X, (*Node_ptr)->rhs_str))
	{
	  return (Prod_insert(X, Left, Pnum, &((*Node_ptr)->left)));
	}
  else if (GT(X, (*Node_ptr)->rhs_str))
	{
	  return (Prod_insert(X, Left, Pnum, &((*Node_ptr)->right)));
	}
  else if (EQUAL(X, (*Node_ptr)->rhs_str))
	{
	  return (*Node_ptr);
	}
  return (NULL);
}




// Search the binary tree of productions to see if the Handle is a valid right 
// hand of side of a production
// This routine is called from Reduce() in the file Po_parse_poly.c, while doing
// the parsing of identity.
// args: X -- a Handle of a reduction.                       
//       Node_ptr -- Pointer to the tree of productions.       
// RETURNS:                                                       
//     If X is a valid handle, return pointer to the node where   
//     the Handle is present.                                     
//     NULL otherwise.                                             
PROD_TREEPTR Prod_member (char X[], PROD_TREEPTR Node_ptr)
{
  if (Node_ptr == NULL)
	{
	  return (NULL);
	}
  else if (EQUAL(X, (Node_ptr)->rhs_str))
	{
	  return (Node_ptr);
	}
  else if (LT(X, Node_ptr->rhs_str))
	{
	  return (Prod_member(X, Node_ptr->left));
	}
  else if (GT(X, Node_ptr->rhs_str))
	{
	  return (Prod_member(X, Node_ptr->right));
	}
  return (NULL);
}




// RETURNS: 1 if string X is less than string Y, 0 otherwise
int LT (char X[], char Y[])
{
  if (strcmp(X, Y) < 0)
	{
	  return (1);
	}
  else
	{
	  return (0);
	}
}



// RETURNS: 1 if string X is greater than string Y, 0 otherwise
int GT (char X[], char Y[])
{
  if (strcmp(X, Y) > 0)
	{
	  return (1);
	}
  else
	{
	  return (0);
	}
}



// RETURNS: 1 if string X is equal to string Y, 0 otherwise
int EQUAL (char X[], char Y[])
{
  if (strcmp (X, Y) == 0)
	{
	  return (1);
	}
  else
	{
	  return (0);
	}
}


// end of Po_prod_bst.cc
