/*
  file:         Ty_routines.cc
  purpose:      routines dealing with Association type of Non_associative words
  created:      David Jacobs circa 1990
  modified:     Pasha Zusmanovich feb 18 2016
  modification: c++ file ext.; cleanup; printf -> iostream
*/

#include <stdlib.h>
#include <iostream>
#include "Po_parse_exptext.hh"
#include "Ty_routines.hh"

int Degree_term (struct term_node*);

static int Catalan[] = {0,1,1,2,5,14,42,132,429,1430,4862,16796,58786,
    208012,742900,2674440};



/* REQUIRES:                                                       */
/*     Pntr -- Pointer to a tree corresponding to the word whose   */
/*         association type is to be computed.                     */ 
/* RETURNS:                                                        */
/*     Association type of the word passed.                        */
/* FUNCTION:                                                       */
/*     Compute the Association type recursively by computing the   */
/*     Association type of the left child and right child.         */
/* NOTE:                                                           */
/*     This routine is called when a command "t word" is issued.   */
int Assoc_number (term_node* Pntr)
{
  int deg1, deg2, t, t1, t2, offset;
  int i,j;

  if (Pntr == NULL)
	{
	  return (0);
	}

  int deg = Degree_term(Pntr);
  if ((deg == 1) || (deg == 2))
	{
	  return (1);
	}
  else 
	{
	  deg1 = Degree_term (Pntr->left);
	  deg2 = Degree_term (Pntr->right);

	  t1 = Assoc_number (Pntr->left);
	  t2 = Assoc_number (Pntr->right);
	  
	  if (deg2 == 1)
		{
		  offset = 0;
		}
	  else 
		{
		  offset = 0;
		  i = deg - 1; j = 1; 
		  while ((i >= (deg1+1)) && (j <= (deg2-1))) 
			{
			  offset = offset + Get_catalan(i) * Get_catalan(j);
			  i--;
			  j++;
            }
        }

	  t = offset + Get_catalan(deg1) * (t2 - 1) + t1;
	  return (t);
    }
} // end of Assoc_number()



/* REQUIRES:                                                       */
/*     Pntr -- Pointer to a tree corresponding to the word whose   */
/*         degree is to be computed.                               */ 
/* RETURNS:                                                        */
/*     Degree of the word passed.                                  */ 
int Degree_term (struct term_node* Pntr)
{ 
  if (Pntr == NULL)
	{
	  return(0);
	}
  if ((Pntr->left == NULL) && (Pntr->right == NULL))
	{
	  return (1);
	}
  else
	{
	  return (Degree_term(Pntr->left) + Degree_term(Pntr->right));
	}
}


/* REQUIRES:                                                       */
/*     i -- index into Catalan table.                              */
/* RETURNS:                                                        */
/*     value of entry i in Catalan table.                          */
int Get_catalan (int i)
{
  if (i <= MAX_DEGREE)
	{
	  return (Catalan[i]);
	}
  else 
	{
	  std::cout << "In Get_catalan(). Degree " << i 
				<< " out of bounds. Exiting." << std::endl;
	  exit (1);
    }
}


// end of Ty_routines.hh

