/*
  file:    scalar.hh
  purpose: definition of Scalar class (base class for implementations of 
           elements of the ground field)
  created: pasha mar 2,8,10 2016
*/

// Scalars (elements of the base field) can be any class which implements 
// the following:
//    unary operator -
//    binary operator +
//    binary operator +=
//    binary operator *
//    binary operator *=
//    unary operator S_inv (inversion)
//    binary operator ==
//    binary operator !=
//    operator << (printing to ostream)
//    conversion from 0 and 1


#ifndef __SCALAR_H
#define __SCALAR_H


template<typename T> class Scalar
{
public:

  friend inline bool operator== (const Scalar<T>& X, const Scalar<T>& Y)
  {
	return (X.val == Y.val);
  };
  
  friend inline bool operator!= (const Scalar<T>& X, const Scalar<T>& Y)
  {
	return (X.val != Y.val);
  };

protected:
  T val;

}; // end of class Scalar




// S_inv() wrapper for GMP rational elements
inline mpq_class S_inv (const mpq_class& q)
{
  if (q == 0)
	{
	  std::cerr << "division by zero in GMP" << std::endl;
	  exit (-1);
	}
  return (1/q);
}



#endif // __SCALAR_H
// end of scalar.hh
